﻿/*
  COPYRIGHT 2022 - PROPERTY OF TOBII AB
  -------------------------------------
  2022 TOBII AB - KARLSROVAGEN 2D, DANDERYD 182 53, SWEDEN - All Rights Reserved.

  NOTICE:  All information contained herein is, and remains, the property of Tobii AB and its suppliers, if any.
  The intellectual and technical concepts contained herein are proprietary to Tobii AB and its suppliers and may be
  covered by U.S.and Foreign Patents, patent applications, and are protected by trade secret or copyright law.
  Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
  permission is obtained from Tobii AB.
*/
using System;
using System.Collections.Generic;
using Tobii.StreamEngine;
using static Tobii.StreamEngine.Interop;

namespace Tobii.StreamEngine
{
    /// <summary>
    /// Implementation of a thin wrapper for communication with Tobii Stream Engine.
    /// Contains no logic and calls directly functions exported from an unmanaged DLL <see cref="StreamEngine.Interop"/>
    /// See full documentation for stream engine functions for more details.
    /// </summary>
    /// <remarks>Wrapper around the stream engine in combination with IStreamEngineWrapper interface allows to easily create a mock of the stream engine for unit tests.</remarks>
    public class StreamEngineWrapper : IStreamEngineWrapper
    {
        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t CalibrationStart(IntPtr device, tobii_enabled_eye_t enabledEye)
        {
            return Interop.tobii_calibration_start(device, enabledEye);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t CalibrationStop(IntPtr device)
        {
            return Interop.tobii_calibration_stop(device);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t WearableConsumerDataSubscribe(IntPtr device, tobii_wearable_consumer_data_callback_t callback, IntPtr userData = default)
        {
            return Interop.tobii_wearable_consumer_data_subscribe(device, callback, userData);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t DeviceProcessCallbacks(IntPtr device)
        {
            return Interop.tobii_device_process_callbacks(device);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t CalibrationComputeAndApply(IntPtr device)
        {
            return Interop.tobii_calibration_compute_and_apply(device);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t CalibrationCollectData3d(IntPtr device, float x, float y, float z)
        {
            return Interop.tobii_calibration_collect_data_3d(device, x, y, z);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t WearableConsumerDataUnsubscribe(IntPtr device)
        {
            return Interop.tobii_wearable_consumer_data_unsubscribe(device);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t DeviceCreate(IntPtr api, string url, tobii_field_of_use_t fieldOfUse, out IntPtr device)
        {
            return Interop.tobii_device_create(api, url, fieldOfUse, out device);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t DeviceCreateEx(IntPtr api, string url, tobii_field_of_use_t fieldOfUse, string[] licenseKeys, List<tobii_license_validation_result_t> licenseResults, out IntPtr device)
        {
            return Interop.tobii_device_create_ex(api, url, fieldOfUse, licenseKeys, licenseResults, out device);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t DeviceDestroy(IntPtr device)
        {
            return Interop.tobii_device_destroy(device);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t ApiCreate(out IntPtr api, tobii_custom_log_t customLog)
        {
            return Interop.tobii_api_create(out api, customLog);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t ApiDestroy(IntPtr api)
        {
            return Interop.tobii_api_destroy(api);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t EnumerateLocalDeviceUrlsInternal(IntPtr api, tobii_device_url_receiver_t receiverFunction, IntPtr userData)
        {
            return Interop.tobii_enumerate_local_device_urls_internal(api, receiverFunction, userData);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t GetDeviceInfo(IntPtr device, out tobii_device_info_t deviceInfo)
        {
            return Interop.tobii_get_device_info(device, out deviceInfo);
        }

        /// <summary>
        /// <inheritdoc/>
        /// </summary>
        public tobii_error_t DeviceReconnect(IntPtr device)
        {
            return Interop.tobii_device_reconnect(device);
        }
    }
}